package utils

import (
	"errors"

	"github.com/go-xorm/xorm"
)

// 系统内置数据种类
const (
	DATATYPE_USER   = "user"
	DATATYPE_CASE   = "case"
	DATATYPE_COURSE = "course"
	DATATYPE_COUPON = "coupon"
	DATATYPE_CARD   = "card"
)

// DataAuthEngine 鉴权数据
type DataAuthEngine struct {
	db *xorm.Engine
}

// NewDataAuthEngine 新建实例
func NewDataAuthEngine(db *xorm.Engine) *DataAuthEngine {
	return &DataAuthEngine{
		db: db,
	}
}

// HasUserPermission 是否有人员权限
func (t *DataAuthEngine) HasUserPermission(userID, borrower string, caseID ...string) error {
	// 1、先判断组织是否相同
	userOrg, err := t.getOrgOfData(userID, DATATYPE_USER)
	if err != nil {
		LogError("查询用户(" + userID + ") 所属组织失败: " + err.Error())
		return errors.New("查询用户相关信息失败")
	}

	borwOrg, err := t.getOrgOfData(borrower, DATATYPE_USER)
	if err != nil {
		LogError("查询用户(" + borrower + ") 所属组织失败: " + err.Error())
		return errors.New("查询用户相关信息失败")
	}

	if userOrg != borwOrg {
		return errors.New("人员不存在或无权浏览")
	}

	// 2、再判断是否案场相同
	sameCases, err := t.getSameCase(userID, borrower)
	if err != nil {
		return errors.New("用户没有相同案场授权")
	}

	// 3、是否指定了案场
	if len(caseID) > 0 {
		if StrSliceIndexOf(sameCases, caseID[0]) < 0 {
			return errors.New("用户没有当前案场权限")
		}
	}

	return nil
}

func (t *DataAuthEngine) getOrgOfData(dataID, dataType string) (string, error) {
	org := make(map[string]interface{})

	switch dataType {
	// 用户
	case DATATYPE_USER:
		if _, err := t.db.SQL("select org_id from sys_user where user_id = ?", dataID).Get(&org); err != nil {
			return "", err
		}

		orgID := org["org_id"].(string)
		return orgID, nil

		// 案场
	case DATATYPE_CASE:
		if _, err := t.db.SQL("select org_id from sys_case where case_id = ?", dataID).Get(&org); err != nil {
			return "", err
		}

		orgID := org["org_id"].(string)
		return orgID, nil
	default:
		return "", nil
	}
}

func (t *DataAuthEngine) getSameCase(user1, user2 string) ([]string, error) {
	sql := `
		SELECT t.case_id
		FROM
			sys_user_case t
		JOIN sys_user_case s ON t.user_id = ?
		AND s.user_id = ?
		AND t.case_id = s.case_id	
	`
	res, err := t.db.Query(sql, user1, user2)
	if err != nil {
		LogError("查询用户相同案场失败: " + err.Error())
		return nil, err
	}

	if res == nil || len(res) == 0 {
		return nil, errors.New("用户没有相同案场授权")
	}

	csIDs := make([]string, 0)
	for _, cs := range res {
		csIDs = append(csIDs, string(cs["case_id"]))
	}

	return csIDs, nil
}
